#!/bin/bash

 # Copyright Jul 30, 2006 by Chris Evich
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
 #   the Free Software Foundation; either version 2 of the License, or
 #   (at your option) any later version.
 #
 #   This program is distributed in the hope that it will be useful,
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 #   GNU General Public License for more details.
 #
 #   You should have received a copy of the GNU General Public License
 #   along with this program; if not, write to the Free Software
 #   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #

####### BEGIN USER CONFIGURATION
MINFILE=5g    # smallest filesize to test
MAXFILE=10g   # largest filesize to test
MINBLK=10m     # smallest blocksize to test
MAXBLK=16m    # largest blocksize to test
IOZONEPATH="/os1/coyle/iozone/iozone3_263/src/current"
####### END USER CONFIGURATION

echo ""

##### CHANGELOG:
# 1.0 -> 1.1 Switched to SYNC tests to increase load, adjusted file/block sizes upward.
# 1.1 -> 1.2 Introduced random delay between executions of iozone
# 1.2 -> 1.3 Commented out call for random delay.

VERSION="1.3"
# -K inject some random access during tests
# -M put uname in the output file
# -o run all tests SYNC, guarantees data is really flushed to disk for each test.
# -+T enable timestamps in logging
IOZONECMD="$IOZONEPATH/iozone -a -K -M -o -+T -n $MINFILE -g $MAXFILE -y $MINBLK -q $MAXBLK -f"

printusage() {
     echo -n "usage: "
     echo "`basename $0` <test1> [test[2-9]] "
     echo ""
     echo "Where testX is the full path to the testfile to use."
}

trap killprocs SIGHUP #incase of a failure signal call killprocs
trap killprocs SIGINT #incase of a failure signal call killprocs
trap killprocs SIGQUIT #incase of a failure signal call killprocs
trap killprocs SIGTERM #incase of a failure signal call killprocs
trap killprocs SIGPIPE #incase of a failure signal call killprocs

killprocs() {
    echo ""
    echo "Killing all iozone processes and exiting."
    killall -9 iozone &> /dev/null
    exit 0
}

randomdelay() {
    let "DELAY=(RANDOM%60)+30" # pick random number between 30 - 90
    echo "Delaying for $DELAY seconds."
    sleep ${DELAY}s
}

if [ -z "$1" ]
then
     printusage
     exit 1
fi

echo "`basename $0` version $VERSION"
echo "Executing iozone tests on $@"
echo "This will run forever.  Press ctrl-c to abort all tests."
echo "Output results contained in /tmp/iozone_output"
echo -n "Testing will begin in 5..."
sleep 1s
echo -n "4..."
sleep 1s
echo -n "3..."
sleep 1s
echo -n "2..."
sleep 1s
echo -n "1..."
sleep 1s
echo ""

echo ""

rm -f /tmp/iozone_process_*_output.log &> /dev/null

while true
do
     echo "-------------------------------------------------------------------"
     let "FILECOUNT=0"
     for filename in $@
     do
          let "FILECOUNT++"
          echo "(`date`) running $IOZONECMD $filename"
          $IOZONECMD $filename >> /tmp/iozone_process_${FILECOUNT}_output.log &
          sleep 2s
          pidof iozone &> /dev/null
          if [ "$?" -gt "0"  ]
          then
               echo "Error executing iozone command, exiting."
               killprocs
          fi
          #randomdelay
     done
     while pidof iozone &> /dev/null
     do
          sleep 5s
     done
done


